/** MID812 and 812 API. API functions for the 812 and 812 boards.
 * Copyright 2012 Sensoray Company Inc.
 * This functions encapsulate the DirectShow functionality of the 812 AVStream driver
 * Note: No support will be given for compiling or modifying the source code of this DLL API.
 */

#ifndef _MID812FUNC_H
#define _MID812FUNC_H

#define MAX_DEVICES 8  // maximum number of 812s in system
#define MAX_STREAMS 8  // number of streams/channels per 812 board

#include "mid812types.h"

/** Opens the SDK/API
  * @return 0 on success
  */
MID812_API int __stdcall S812_OpenBoard(int devid);

/** Cleans up and shuts down the API gracefully
 *  @return 0 on success
 */
MID812_API int __stdcall S812_CloseBoard(int devid); 


// query number of devices in system
MID812_API int __stdcall S812_GetNumDevices(int *pNumDevices);


/** set the current video system.  If board is currently streaming 
 *  board must be stopped and restarted before it takes affect.
 *  @param vidsys is the video system type
 *  @return 0 on success
 */
MID812_API int __stdcall S812_SetVidSys(MID812_VIDSYS vidsys, int devid, int strmidx);

MID812_API int __stdcall S812_GetVidSys(MID812_VIDSYS *vidsys, int devid, int strmidx);


/** sets the colorspace.  Mostly for callback function.
 *  Using RGB565 with a video card that does not support native RGB565 rendering
 *  may affect preview performance.
 * If board is currently streaming 
 *  board must be stopped and restarted before it takes affect. *  @return 0 on success
 */
MID812_API int __stdcall S812_SetColorSpace(MID812_COLORSPACE cspace, int devid, int strmidx);

MID812_API int __stdcall S812_GetColorSpace(MID812_COLORSPACE *cspace, int devid, int strmidx);


/** Set the window for the video streaming per device
 *  @param hwnd pointer to video display window
 *  @param devid: device id(board and channel)
 *  Note: this setting will override S812_SetRenderVideo and turn on 
 *  Video rendering(display) as it is assumed setting a window means display is wanted
 */
MID812_API int __stdcall S812_SetPreviewWindow( HWND hwnd, int devid, int strmidx);

/** set the video position in the clipping window 
 *  @param xpos x position of video
 *  @param ypos y position of video
 *  @param xsize size of video display
 *  @param ysize size of video display
 *  @return 0 on success
 */ 
MID812_API int __stdcall S812_SetVideoPosition(int xpos, int ypos, int xsize, int ysize, int devid, int strmidx);


/** starts the preview of video and audio(if selected)
  * @param devid is the board/channelid instance in the system
  */
MID812_API int __stdcall S812_StartPreview(int devid, int strmidx);


/** 
 * Starts recording to file using filter graph previously set up.
 * @param filename is the video/mux filename to record to.  Use the full path and extension
 * @return 0 on success
 */
MID812_API int __stdcall S812_StartRecord(const char *filename, int devid, int strmidx);
// unicode
MID812_API int __stdcall S812_StartRecordW(const wchar_t *filename, int devid, int strmidx);

// unicode
MID812_API int __stdcall S812_StartPreviewAndRecordW(const wchar_t *filename, int devid, int strmidx);
// non-unicode
MID812_API int __stdcall S812_StartPreviewAndRecord(const char *filename, int devid, int strmidx);


/** stops preview, record or callback streaming
 */
MID812_API int __stdcall S812_StopStream(int devid, int strmidx);


MID812_API int __stdcall S812_StartAudioPreview(int devid, int strmidx);
/** stops preview, record or callback streaming
 */
MID812_API int __stdcall S812_StopAudioPreview(int devid, int strmidx);




/////////////////////////////////////////////////
// General configuration options

/** set the brightness, contrast and saturation
*   @param param is the parameter to change(MID812_LEVEL_CONTRAST, MID812_LEVEL_BRIGHTNESS, MID812_LEVEL_SATURATION, MID812_LEVEL_HUE)
*   @param value is the value of the parameter
*    ( BRIGHTNESS: 0 to 255 default 128)
*    ( SATURATION: -127 to 127 default 64)
*    ( CONTRAST: -127 to 127 default 68 (negative values not recommended))
*    ( HUE: -127 to 127, default 0 (recommended to stay unchanged))
*   @return 0 on success
*/
MID812_API int __stdcall S812_SetLevel(int param, int value, int devid, int strmidx);

/** get the brightness, contrast or saturation
  * @param param is the parameter to get(MID812_LEVEL_CONTRAST, MID812_LEVEL_BRIGHTNESS, MID812_LEVEL_SATURATION, MID812_LEVEL_HUE)
  * @param value is the returned value of the parameter
  * @return 0 on success
  */
MID812_API int __stdcall S812_GetLevel(int param, int *value, int devid, int strmidx);

// lines is not valid if viddetect is 0
// otherwise 525 for NTSC, 625 for PAL
MID812_API int __stdcall S812_GetVidStatus(int *viddetect, int *lines, int devid, int strmidx);

/** Set the video size.  Must be done when stream is stopped.
 *  @param enum eVidSize see vidsizes
 *  @return 0 on success
 */
MID812_API int __stdcall S812_SetVidSize(MID812_RES eVidSize, int devid, int strmidx);

MID812_API int __stdcall S812_GetVidSize(MID812_RES *iVidSize, int devid, int strmidx);

////////////////////////////////////////////////////////////////////////
// Callback functions from application
/** Repaint callback should be called when application receives a WM_PAINT event.  This is necessary to
 *  notify the video renderer of a repaint event. 
 *  @param hdc device context handle or NULL if unknown
 *  @return 0 on success
 */
MID812_API int __stdcall S812_Repaint( HDC hdc, int devid, int strmidx);

/** Informs the video renderer that a WM_DISPLAYCHANGE message has been received by the application
 *  This callback should be called when the application has a WM_DISPLAYCHANGE event
 *  @return 0 on success
 */
MID812_API int __stdcall S812_DisplayChange(int devid, int strmidx);
// end callback functions

// returns frame in pFrame.
// snapshots must be turned on
MID812_API int __stdcall S812_GetFrame( long bufSize, unsigned char *pFrame, BITMAPINFOHEADER *pBMI, int devid, int strmidx);
/** takes a snapshot and saves to file in filename.  
*  Note : JPEGs are compressed by SW in Windows, not by the hardware.
*         BMPs will be more CPU efficient but take more space
*   @param szFilename is the fully qualified file + path( without extension)
*   @param filetype is the file type to save(MID812_FILE_JPEG and/or MID812_FILE_BMP)
*   @param qual jpeg quality, (use 25-100, default 100).
*   @param devid: device id(board and channel)
*/ 
// snapshots must be turned on.
MID812_API int __stdcall S812_SnapshotToFile( const char *szFilename, int filetype, int qual, int devid, int strmidx);
// unicode version
MID812_API int __stdcall S812_SnapshotToFileW( const wchar_t *szFilename, int filetype, int qual, int devid, int strmidx);

// "Frame Grabber" mode.  Captures all frames to a callback function.
//
typedef int (__stdcall *cbfunc_t) ( BYTE *merged, long size, int devid, int strmidx);
MID812_API int __stdcall S812_StartCallback(int devid, int strmidx);
MID812_API int __stdcall S812_RegisterCallback(cbfunc_t pfunc, cbfunc_t pfunc_aud, int devid, int strmidx);
// snapshots enable/disable
// On some computers, snapshots may degrade preview performance.
// Must be set before stream starts to take effect
MID812_API int __stdcall S812_EnableSnapshots(int devid, int strmidx, BOOL bOn);
// advanced directshow settings
// video renderer type, add "decompressor" or not (decomp is recommended for smoother video)
// Must be set before stream starts to take effect.  Optional function. Defaults are recommended (without calling this function)
// defaults are iDecomp==1 and MID812_VMR9
// iDecomp == 0 is no decompress filter (not officially supported)
// iDecomp == 1 is decompress intermediate filter (av demux)
// iDecomp == 2 is motion adaptive de-interlace filter
MID812_API int __stdcall S812_SetDirectShow(int devid, int strmidx, int iDecomp, MID812_DSHOW rend);

// same as S812_SetDirectShow, but with consistent SDK parameter order
MID812_API int __stdcall S812_SetDShow(int iDecomp, MID812_DSHOW rend, int devid, int strmidx);
MID812_API int __stdcall S812_GetDShow(int *iDecomp, MID812_DSHOW *rend, int devid, int strmidx);

// flip the image (mirror/horizontal or vertical)
MID812_API int __stdcall S812_FlipImage(BOOL bFlipH, BOOL bFlipV, int devid, int strmidx);
MID812_API int __stdcall S812_SetFlipImage(BOOL bFlipH, BOOL bFlipV, int devid, int strmidx); // same as S812_FlipImage
MID812_API int __stdcall S812_GetFlipImage(BOOL *bFlipH, BOOL *bFlipV, int devid, int strmidx);


// returns last error code if any for debug.  supplemental to return value of other functions
MID812_API int __stdcall S812_GetDebugErrorCode(int devid, int strmidx);

// returns additional error code if any for debug.  supplemental to return value of other functions
MID812_API int __stdcall S812_GetDebugErrorCode2();

MID812_API int __stdcall S812_GetDLLVersion(int *v1, int *v2, int *v3);


MID812_API int __stdcall S812_GetDriverVersion(int *v1, int *v2, int *v3, int *v4, int devid, int strmidx);


// If using non-compliant DirectShow app that doesn't properly set the standard and 
// the user never switches the video standard of the connected camera.
// fixed == 0, normal driver
// fixed == 1, NTSC only driver
// fixed == 2, PAL only driver
MID812_API int __stdcall S812_SetFixedStandard(int fixed, int devid);
MID812_API int __stdcall S812_GetFixedStandard(int *fixed, int devid);

#endif
